@@ -2,17 +2,7 @@
 #
 # Copyright 2016 Hewlett-Packard Development Company, L.P.
 #
-# Licensed under the Apache License, Version 2.0 (the "License"); you may
-# not use this file except in compliance with the License. You may obtain
-# a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-# License for the specific language governing permissions and limitations
-# under the License.
+# SPDX-License-Identifier: Apache-2.0
 
 r"""
 ====================================================
@@ -27,7 +17,8 @@ ast.Call nodes.
 B301: pickle
 ------------
 
-Pickle library appears to be in use, possible security issue.
+Pickle and modules that wrap it can be unsafe when used to
+deserialize untrusted data, possible security issue.
 
 +------+---------------------+------------------------------------+-----------+
 | ID   |  Name               |  Calls                             |  Severity |
@@ -38,6 +29,11 @@ Pickle library appears to be in use, possible security issue.
 |      |                     | - cPickle.loads                    |           |
 |      |                     | - cPickle.load                     |           |
 |      |                     | - cPickle.Unpickler                |           |
+|      |                     | - dill.loads                       |           |
+|      |                     | - dill.load                        |           |
+|      |                     | - dill.Unpickler                   |           |
+|      |                     | - shelve.open                      |           |
+|      |                     | - shelve.DbfilenameShelf           |           |
 +------+---------------------+------------------------------------+-----------+
 
 B302: marshal
@@ -55,17 +51,25 @@ Deserialization with the marshal module is possibly dangerous.
 B303: md5
 ---------
 
-Use of insecure MD2, MD4, or MD5 hash function.
+Use of insecure MD2, MD4, MD5, or SHA1 hash function.
 
 +------+---------------------+------------------------------------+-----------+
 | ID   |  Name               |  Calls                             |  Severity |
 +======+=====================+====================================+===========+
 | B303 | md5                 | - hashlib.md5                      | Medium    |
+|      |                     | - hashlib.sha1                     |           |
 |      |                     | - Crypto.Hash.MD2.new              |           |
 |      |                     | - Crypto.Hash.MD4.new              |           |
 |      |                     | - Crypto.Hash.MD5.new              |           |
+|      |                     | - Crypto.Hash.SHA.new              |           |
+|      |                     | - Cryptodome.Hash.MD2.new          |           |
+|      |                     | - Cryptodome.Hash.MD4.new          |           |
+|      |                     | - Cryptodome.Hash.MD5.new          |           |
+|      |                     | - Cryptodome.Hash.SHA.new          |           |
 |      |                     | - cryptography.hazmat.primitives   |           |
 |      |                     |   .hashes.MD5                      |           |
+|      |                     | - cryptography.hazmat.primitives   |           |
+|      |                     |   .hashes.SHA1                     |           |
 +------+---------------------+------------------------------------+-----------+
 
 B304 - B305: ciphers and modes
@@ -82,6 +86,11 @@ as AES.
 |      |                     | - Crypto.Cipher.Blowfish.new       |           |
 |      |                     | - Crypto.Cipher.DES.new            |           |
 |      |                     | - Crypto.Cipher.XOR.new            |           |
+|      |                     | - Cryptodome.Cipher.ARC2.new       |           |
+|      |                     | - Cryptodome.Cipher.ARC4.new       |           |
+|      |                     | - Cryptodome.Cipher.Blowfish.new   |           |
+|      |                     | - Cryptodome.Cipher.DES.new        |           |
+|      |                     | - Cryptodome.Cipher.XOR.new        |           |
 |      |                     | - cryptography.hazmat.primitives   |           |
 |      |                     |   .ciphers.algorithms.ARC4         |           |
 |      |                     | - cryptography.hazmat.primitives   |           |
@@ -130,8 +139,8 @@ be reviewed.
 B309: httpsconnection
 ---------------------
 
-Use of HTTPSConnection does not provide security, see
-https://wiki.openstack.org/wiki/OSSN/OSSN-0033
+Use of HTTPSConnection on older versions of Python prior to 2.7.9 and 3.4.3 do
+not provide security, see https://wiki.openstack.org/wiki/OSSN/OSSN-0033
 
 +------+---------------------+------------------------------------+-----------+
 | ID   |  Name               |  Calls                             |  Severity |
@@ -203,7 +212,7 @@ B313 - B320: XML
 ----------------
 
 Most of this is based off of Christian Heimes' work on defusedxml:
-https://pypi.python.org/pypi/defusedxml/#defusedxml-sax
+https://pypi.org/project/defusedxml/#defusedxml-sax
 
 Using various XLM methods to parse untrusted XML data is known to be vulnerable
 to XML attacks. Methods should be replaced with their defusedxml equivalents.
@@ -236,9 +245,6 @@ to XML attacks. Methods should be replaced with their defusedxml equivalents.
 | B319 | xml_bad_pulldom     | - xml.dom.pulldom.parse            | Medium    |
 |      |                     | - xml.dom.pulldom.parseString      |           |
 +------+---------------------+------------------------------------+-----------+
-| B319 | xml_bad_pulldom     | - xml.dom.pulldom.parse            | Medium    |
-|      |                     | - xml.dom.pulldom.parseString      |           |
-+------+---------------------+------------------------------------+-----------+
 | B320 | xml_bad_etree       | - lxml.etree.parse                 | Medium    |
 |      |                     | - lxml.etree.fromstring            |           |
 |      |                     | - lxml.etree.RestrictedElement     |           |
@@ -264,7 +270,7 @@ B322: input
 
 The input method in Python 2 will read from standard input, evaluate and
 run the resulting string as python source code. This is similar, though in
-many ways worse, then using eval. On Python 2, use raw_input instead, input
+many ways worse, than using eval. On Python 2, use raw_input instead, input
 is safe in Python 3.
 
 +------+---------------------+------------------------------------+-----------+
@@ -273,6 +279,38 @@ is safe in Python 3.
 | B322 | input               | - input                            | High      |
 +------+---------------------+------------------------------------+-----------+
 
+B323: unverified_context
+------------------------
+
+By default, Python will create a secure, verified ssl context for use in such
+classes as HTTPSConnection. However, it still allows using an insecure
+context via the _create_unverified_context that reverts to the previous
+behavior that does not validate certificates or perform hostname checks.
+
++------+---------------------+------------------------------------+-----------+
+| ID   |  Name               |  Calls                             |  Severity |
++======+=====================+====================================+===========+
+| B323 | unverified_context  | - ssl._create_unverified_context   | Medium    |
++------+---------------------+------------------------------------+-----------+
+
+B325: tempnam
+--------------
+
+Use of os.tempnam() and os.tmpnam() is vulnerable to symlink attacks. Consider
+using tmpfile() instead.
+
+For further information:
+    https://docs.python.org/2.7/library/os.html#os.tempnam
+    https://docs.python.org/3/whatsnew/3.0.html?highlight=tempnam
+    https://bugs.python.org/issue17880
+
++------+---------------------+------------------------------------+-----------+
+| ID   |  Name               |  Calls                             |  Severity |
++======+=====================+====================================+===========+
+| B325 | tempnam             | - os.tempnam                       | Medium    |
+|      |                     | - os.tmpnam                        |           |
++------+---------------------+------------------------------------+-----------+
+
 """
 
 from bandit.blacklists import utils
@@ -298,8 +336,14 @@ def gen_blacklist():
          'pickle.Unpickler',
          'cPickle.loads',
          'cPickle.load',
-         'cPickle.Unpickler'],
-        'Pickle library appears to be in use, possible security issue.'
+         'cPickle.Unpickler',
+         'dill.loads',
+         'dill.load',
+         'dill.Unpickler',
+         'shelve.open',
+         'shelve.DbfilenameShelf'],
+        'Pickle and modules that wrap it can be unsafe when used to '
+        'deserialize untrusted data, possible security issue.'
         ))
 
     sets.append(utils.build_conf_dict(
@@ -310,11 +354,18 @@ def gen_blacklist():
     sets.append(utils.build_conf_dict(
         'md5', 'B303',
         ['hashlib.md5',
+         'hashlib.sha1',
          'Crypto.Hash.MD2.new',
          'Crypto.Hash.MD4.new',
          'Crypto.Hash.MD5.new',
-         'cryptography.hazmat.primitives.hashes.MD5'],
-        'Use of insecure MD2, MD4, or MD5 hash function.'
+         'Crypto.Hash.SHA.new',
+         'Cryptodome.Hash.MD2.new',
+         'Cryptodome.Hash.MD4.new',
+         'Cryptodome.Hash.MD5.new',
+         'Cryptodome.Hash.SHA.new',
+         'cryptography.hazmat.primitives.hashes.MD5',
+         'cryptography.hazmat.primitives.hashes.SHA1'],
+        'Use of insecure MD2, MD4, MD5, or SHA1 hash function.'
         ))
 
     sets.append(utils.build_conf_dict(
@@ -324,6 +375,11 @@ def gen_blacklist():
          'Crypto.Cipher.Blowfish.new',
          'Crypto.Cipher.DES.new',
          'Crypto.Cipher.XOR.new',
+         'Cryptodome.Cipher.ARC2.new',
+         'Cryptodome.Cipher.ARC4.new',
+         'Cryptodome.Cipher.Blowfish.new',
+         'Cryptodome.Cipher.DES.new',
+         'Cryptodome.Cipher.XOR.new',
          'cryptography.hazmat.primitives.ciphers.algorithms.ARC4',
          'cryptography.hazmat.primitives.ciphers.algorithms.Blowfish',
          'cryptography.hazmat.primitives.ciphers.algorithms.IDEA'],
@@ -360,7 +416,8 @@ def gen_blacklist():
         ['httplib.HTTPSConnection',
          'http.client.HTTPSConnection',
          'six.moves.http_client.HTTPSConnection'],
-        'Use of HTTPSConnection does not provide security, see '
+        'Use of HTTPSConnection on older versions of Python prior to 2.7.9 '
+        'and 3.4.3 do not provide security, see '
         'https://wiki.openstack.org/wiki/OSSN/OSSN-0033'
         ))
 
@@ -405,7 +462,7 @@ def gen_blacklist():
         ))
 
     # Most of this is based off of Christian Heimes' work on defusedxml:
-    #   https://pypi.python.org/pypi/defusedxml/#defusedxml-sax
+    #   https://pypi.org/project/defusedxml/#defusedxml-sax
 
     xml_msg = ('Using {name} to parse untrusted XML data is known to be '
                'vulnerable to XML attacks. Replace {name} with its '
@@ -490,9 +547,26 @@ def gen_blacklist():
         'input', 'B322', ['input'],
         'The input method in Python 2 will read from standard input, '
         'evaluate and run the resulting string as python source code. This '
-        'is similar, though in many ways worse, then using eval. On Python '
+        'is similar, though in many ways worse, than using eval. On Python '
         '2, use raw_input instead, input is safe in Python 3.',
         'HIGH'
         ))
 
+    sets.append(utils.build_conf_dict(
+        'unverified_context', 'B323', ['ssl._create_unverified_context'],
+        'By default, Python will create a secure, verified ssl context for '
+        'use in such classes as HTTPSConnection. However, it still allows '
+        'using an insecure context via the _create_unverified_context that '
+        'reverts to the previous behavior that does not validate certificates '
+        'or perform hostname checks.'
+        ))
+
+    # skipped B324 (used in bandit/plugins/hashlib_new_insecure_functions.py)
+
+    sets.append(utils.build_conf_dict(
+        'tempnam', 'B325', ['os.tempnam', 'os.tmpnam'],
+        'Use of os.tempnam() and os.tmpnam() is vulnerable to symlink '
+        'attacks. Consider using tmpfile() instead.'
+        ))
+
     return {'Call': sets}
